#include "mainwindow.h"
#include <QInputDialog>
#include <QMessageBox>
#include <QStringList>

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent), centralWidget(new QWidget(this)), mainLayout(new QVBoxLayout),
    buttonLayout(new QHBoxLayout), queueSelectionLayout(new QHBoxLayout), operationLayout(new QHBoxLayout),
    addButton(new QPushButton("Add")), deleteButton(new QPushButton("Delete")),
    getMaxButton(new QPushButton("Get Max")), getMinButton(new QPushButton("Get Min")),
    clearButton(new QPushButton("Clear")), addQueueButton(new QPushButton("Add Queue")),
    addQueuesButton(new QPushButton("Add Queues")), subtractQueuesButton(new QPushButton("Subtract Queues")),
    overflowMemoryButton(new QPushButton("Overflow Memory")), displayQueueButton(new QPushButton("Display Queue")),
    queueDisplay(new QLabel("Queue: ")), inputSequence(new QLineEdit), queueSelector(new QComboBox), queueOutput(new QTextEdit),
    currentQueueIndex(0) {

    setCentralWidget(centralWidget);
    centralWidget->setLayout(mainLayout);

    queueSelectionLayout->addWidget(new QLabel("Select Queue:"));
    queueSelectionLayout->addWidget(queueSelector);
    queueSelectionLayout->addWidget(addQueueButton);

    buttonLayout->addWidget(addButton);
    buttonLayout->addWidget(deleteButton);
    buttonLayout->addWidget(getMaxButton);
    buttonLayout->addWidget(getMinButton);
    buttonLayout->addWidget(clearButton);

    operationLayout->addWidget(addQueuesButton);
    operationLayout->addWidget(subtractQueuesButton);
    operationLayout->addWidget(overflowMemoryButton);
    operationLayout->addWidget(displayQueueButton);

    mainLayout->addLayout(queueSelectionLayout);
    mainLayout->addWidget(queueDisplay);
    mainLayout->addWidget(inputSequence);
    mainLayout->addLayout(buttonLayout);
    mainLayout->addLayout(operationLayout);
    mainLayout->addWidget(queueOutput);

    queueOutput->setReadOnly(true);

    connect(addButton, &QPushButton::clicked, this, &MainWindow::onAddButtonClicked);
    connect(deleteButton, &QPushButton::clicked, this, &MainWindow::onDeleteButtonClicked);
    connect(getMaxButton, &QPushButton::clicked, this, &MainWindow::onGetMaxButtonClicked);
    connect(getMinButton, &QPushButton::clicked, this, &MainWindow::onGetMinButtonClicked);
    connect(clearButton, &QPushButton::clicked, this, &MainWindow::onClearButtonClicked);
    connect(addQueueButton, &QPushButton::clicked, this, &MainWindow::onAddQueueButtonClicked);
    connect(queueSelector, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &MainWindow::onQueueSelectionChanged);
    connect(addQueuesButton, &QPushButton::clicked, this, &MainWindow::onAddQueuesButtonClicked);
    connect(subtractQueuesButton, &QPushButton::clicked, this, &MainWindow::onSubtractQueuesButtonClicked);
    connect(overflowMemoryButton, &QPushButton::clicked, this, &MainWindow::onOverflowMemoryButtonClicked);
    connect(displayQueueButton, &QPushButton::clicked, this, &MainWindow::onDisplayQueueButtonClicked);

    queues.emplace_back();
    queueSelector->addItem("Queue 1");
    updateQueueDisplay();
}

MainWindow::~MainWindow() {}

void MainWindow::onAddButtonClicked() {
    QString sequence = inputSequence->text();
    QStringList values = sequence.split(" ", Qt::SkipEmptyParts);

    for (const QString& value : values) {
        bool ok;
        double numValue = value.toDouble(&ok);
        if (ok) {
            try {
                queues[currentQueueIndex].addtoqueue(numValue);
            } catch (const std::exception& e) {
                displayError(e);
                return;
            }
        } else {
            QMessageBox::warning(this, "Input Error", "Invalid input. Please enter valid numbers separated by spaces.");
            return;
        }
    }

    updateQueueDisplay();
    inputSequence->clear();
}

void MainWindow::onDeleteButtonClicked() {
    try {
        queues[currentQueueIndex].deletefromqueue();
        updateQueueDisplay();
    } catch (const std::exception& e) {
        displayError(e);
    }
}

void MainWindow::onGetMaxButtonClicked() {
    try {
        double max = queues[currentQueueIndex].getMax();
        QMessageBox::information(this, "Max Value", QString("Max: %1").arg(max));
    } catch (const std::exception& e) {
        displayError(e);
    }
}

void MainWindow::onGetMinButtonClicked() {
    try {
        double min = queues[currentQueueIndex].getMin();
        QMessageBox::information(this, "Min Value", QString("Min: %1").arg(min));
    } catch (const std::exception& e) {
        displayError(e);
    }
}

void MainWindow::onClearButtonClicked() {
    queues[currentQueueIndex].clear();
    updateQueueDisplay();
}

void MainWindow::onAddQueueButtonClicked() {
    int newQueueIndex = queues.size();
    queues.emplace_back();
    queueSelector->addItem(QString("Queue %1").arg(newQueueIndex + 1));
    queueSelector->setCurrentIndex(newQueueIndex);
}

void MainWindow::onQueueSelectionChanged(int index) {
    currentQueueIndex = index;
    updateQueueDisplay();
}

void MainWindow::onAddQueuesButtonClicked() {
    bool ok;
    int queue1Index = QInputDialog::getInt(this, "Select First Queue", "Enter the index of the first queue (starting from 1):", 1, 1, queues.size(), 1, &ok);
    if (!ok) return;
    int queue2Index = QInputDialog::getInt(this, "Select Second Queue", "Enter the index of the second queue (starting from 1):", 1, 1, queues.size(), 1, &ok);
    if (!ok) return;

    queue1Index -= 1;
    queue2Index -= 1;

    try {
        CQueue resultQueue = queues[queue1Index] + queues[queue2Index];
        queues.push_back(resultQueue);
        queueSelector->addItem(QString("Queue %1").arg(queues.size()));
        queueSelector->setCurrentIndex(queues.size() - 1);
        updateQueueDisplay();
    } catch (const std::exception& e) {
        displayError(e);
    }
}

void MainWindow::onSubtractQueuesButtonClicked() {
    bool ok;
    int queue1Index = QInputDialog::getInt(this, "Select First Queue", "Enter the index of the first queue (starting from 1):", 1, 1, queues.size(), 1, &ok);
    if (!ok) return;
    int queue2Index = QInputDialog::getInt(this, "Select Second Queue", "Enter the index of the second queue (starting from 1):", 1, 1, queues.size(), 1, &ok);
    if (!ok) return;

    queue1Index -= 1;
    queue2Index -= 1;

    try {
        CQueue resultQueue = queues[queue1Index] - queues[queue2Index];
        queues.push_back(resultQueue);
        queueSelector->addItem(QString("Queue %1").arg(queues.size()));
        queueSelector->setCurrentIndex(queues.size() - 1);
        updateQueueDisplay();
    } catch (const std::exception& e) {
        displayError(e);
    }
}

void MainWindow::onOverflowMemoryButtonClicked() {
    try {
        while (true) {
            queues[currentQueueIndex].addtoqueue(1.0);
        }
    } catch (const std::exception& e) {
        displayError(e);
    }
}

void MainWindow::onDisplayQueueButtonClicked() {
    try {
        if (queues[currentQueueIndex].isEmpty()) {
            throw std::runtime_error("Queue is empty");
        }

        QStringList elements;
        for (int i = 0; i < queues[currentQueueIndex].getSize(); ++i) {
            elements.append(QString::number(queues[currentQueueIndex].getQueueElement(i)));
        }
        queueOutput->setText(elements.join(", "));
    } catch (const std::exception& e) {
        displayError(e);
    }
}

void MainWindow::updateQueueDisplay() {
    QStringList elements;
    for (int i = 0; i < queues[currentQueueIndex].getSize(); ++i) {
        elements.append(QString::number(queues[currentQueueIndex].getQueueElement(i)));
    }
    queueDisplay->setText("Queue: " + elements.join(", "));
}

void MainWindow::displayError(const std::exception& e) {
    QMessageBox::critical(this, "Error", e.what());
}
